@@ -33,10 +33,6 @@ struct priv {
     AudioUnit audio_unit;
 
     uint64_t hw_latency_us;
-
-    // options
-    int opt_device_id;
-    int opt_list;
 };
 
 bool ca_layout_to_mp_chmap(struct ao *ao, AudioChannelLayout *layout,
@@ -149,11 +145,15 @@ static bool init_audiounit(struct ao *ao, AudioStreamBasicDescription asbd);
 
 static int init(struct ao *ao)
 {
+    if (AF_FORMAT_IS_IEC61937(ao->format)) {
+        MP_WARN(ao, "detected IEC61937, redirecting to coreaudio_exclusive\n");
+        ao->redirect = "coreaudio_exclusive";
+        return CONTROL_ERROR;
+    }
+
     struct priv *p = ao->priv;
 
-    if (p->opt_list) ca_print_device_list(ao);
-
-    OSStatus err = ca_select_device(ao, p->opt_device_id, &p->device);
+    OSStatus err = ca_select_device(ao, ao->device, &p->device);
     CHECK_CA_ERROR("failed to select device");
 
     if (!init_chmap(ao))
@@ -185,7 +185,7 @@ static bool init_chmap(struct ao *ao)
                        &layouts, &n_layouts);
     CHECK_CA_ERROR("could not get audio device prefered layouts");
 
-    struct mp_chmap_sel chmap_sel = {0};
+    struct mp_chmap_sel chmap_sel = {.tmp = p};
     for (int i = 0; i < n_layouts; i++) {
         struct mp_chmap chmap = {0};
         if (ca_layout_to_mp_chmap(ao, &layouts[i], &chmap))
@@ -221,9 +221,9 @@ static bool init_audiounit(struct ao *ao, AudioStreamBasicDescription asbd)
 
     AudioComponentDescription desc = (AudioComponentDescription) {
         .componentType         = kAudioUnitType_Output,
-        .componentSubType      = (p->opt_device_id < 0) ?
-                                    kAudioUnitSubType_DefaultOutput :
-                                    kAudioUnitSubType_HALOutput,
+        .componentSubType      = (ao->device) ?
+                                    kAudioUnitSubType_HALOutput :
+                                    kAudioUnitSubType_DefaultOutput,
         .componentManufacturer = kAudioUnitManufacturer_Apple,
         .componentFlags        = 0,
         .componentFlagsMask    = 0,
@@ -440,10 +440,6 @@ const struct ao_driver audio_out_coreaudio = {
     .control   = control,
     .pause     = stop,
     .resume    = start,
+    .list_devs = ca_get_device_list,
     .priv_size = sizeof(struct priv),
-    .options = (const struct m_option[]) {
-        OPT_INT("device_id", opt_device_id, 0, OPTDEF_INT(-1)),
-        OPT_FLAG("list", opt_list, 0),
-        {0}
-    },
 };
