@@ -37,6 +37,8 @@
 #include "osdep/io.h"
 #include "osdep/timer.h"
 
+#include "libmpv/client.h"
+
 #include "msg.h"
 #include "msg_control.h"
 
@@ -51,7 +53,6 @@ struct mp_log_root {
     bool module;
     bool show_time;
     bool termosd;       // use terminal control codes for status line
-    bool header;        // indicate that message header should be printed
     int blank_lines;    // number of lines useable by status
     int status_lines;   // number of current status lines
     bool color;
@@ -67,6 +68,8 @@ struct mp_log_root {
      * (This is perhaps better than maintaining a globally accessible and
      * synchronized mp_log tree.) */
     atomic_ulong reload_counter;
+    // --- protected by mp_msg_lock
+    char buffer[MSGSIZE_MAX];
 };
 
 struct mp_log {
@@ -155,6 +158,9 @@ static void prepare_status_line(struct mp_log_root *root, char *new_status)
     }
 
     size_t old_lines = root->status_lines;
+    if (!new_status[0] && old_lines == 0)
+        return; // nothing to clear
+
     size_t clear_lines = MPMIN(MPMAX(new_lines, old_lines), root->blank_lines);
 
     // clear the status line itself
@@ -229,70 +235,42 @@ static void pretty_print_module(FILE* stream, const char *prefix, bool use_color
         set_msg_color(stream, lev);
 }
 
-static void print_msg_on_terminal(struct mp_log *log, int lev, char *text)
+static bool test_terminal_level(struct mp_log *log, int lev)
 {
+    return lev <= log->terminal_level &&
+           !(lev == MSGL_STATUS && terminal_in_background());
+}
+
+static void print_terminal_line(struct mp_log *log, int lev, char *text)
+{
+    if (!test_terminal_level(log, lev))
+        return;
+
     struct mp_log_root *root = log->root;
     FILE *stream = (root->force_stderr || lev == MSGL_STATUS) ? stderr : stdout;
 
-    if (!(lev <= log->terminal_level))
-        return;
-
-    bool header = root->header;
-    const char *prefix = log->prefix;
-    char *terminate = NULL;
-
-    if ((lev >= MSGL_V) || root->verbose || root->module)
-        prefix = log->verbose_prefix;
-
-    if (lev == MSGL_STATUS) {
-        // skip status line output if stderr is a tty but in background
-        if (terminal_in_background())
-            return;
-        // don't clear if we don't have to
-        if (!text[0] && !root->status_lines)
-            return;
-        if (root->termosd) {
-            prepare_status_line(root, text);
-            terminate = "\r";
-        } else {
-            terminate = "\n";
-        }
-        root->header = true;
-    } else {
+    if (lev != MSGL_STATUS)
         flush_status_line(root);
-        size_t len = strlen(text);
-        root->header = len && text[len - 1] == '\n';
-        if (lev == MSGL_STATS)
-            terminate = "\n";
-    }
 
     if (root->color)
         set_msg_color(stream, lev);
 
-    do {
-        if (header) {
-            if (root->show_time)
-                fprintf(stream, "[%" PRId64 "] ", mp_time_us());
+    if (root->show_time)
+        fprintf(stream, "[%" PRId64 "] ", mp_time_us());
 
-            if (prefix) {
-                if (root->module) {
-                    pretty_print_module(stream, prefix, root->color, lev);
-                } else {
-                    fprintf(stream, "[%s] ", prefix);
-                }
-            }
+    const char *prefix = log->prefix;
+    if ((lev >= MSGL_V) || root->verbose || root->module)
+        prefix = log->verbose_prefix;
+
+    if (prefix) {
+        if (root->module) {
+            pretty_print_module(stream, prefix, root->color, lev);
+        } else {
+            fprintf(stream, "[%s] ", prefix);
         }
+    }
 
-        char *next = strchr(text, '\n');
-        int len = next ? next - text + 1 : strlen(text);
-        fprintf(stream, "%.*s", len, text);
-        text = text + len;
-
-        header = true;
-    } while (text[0]);
-
-    if (terminate)
-        fprintf(stream, "%s", terminate);
+    fprintf(stream, "%s", text);
 
     if (root->color)
         set_term_color(stream, -1);
@@ -304,7 +282,7 @@ static void write_msg_to_buffers(struct mp_log *log, int lev, char *text)
     struct mp_log_root *root = log->root;
     for (int n = 0; n < root->num_buffers; n++) {
         struct mp_log_buffer *buffer = root->buffers[n];
-        if (lev <= buffer->level) {
+        if (lev <= buffer->level && lev != MSGL_STATUS) {
             // Assuming a single writer (serialized by msg lock)
             int avail = mp_ring_available(buffer->ring) / sizeof(void *);
             if (avail < 1)
@@ -321,7 +299,7 @@ static void write_msg_to_buffers(struct mp_log *log, int lev, char *text)
                 *entry = (struct mp_log_buffer_entry) {
                     .prefix = "overflow",
                     .level = MSGL_FATAL,
-                    .text = "",
+                    .text = "log message buffer overflow\n",
                 };
             }
             mp_ring_write(buffer->ring, (unsigned char *)&entry, sizeof(entry));
@@ -348,15 +326,54 @@ void mp_msg_va(struct mp_log *log, int lev, const char *format, va_list va)
     pthread_mutex_lock(&mp_msg_lock);
 
     char tmp[MSGSIZE_MAX];
-    if (vsnprintf(tmp, MSGSIZE_MAX, format, va) < 0)
-        snprintf(tmp, MSGSIZE_MAX, "[fprintf error]\n");
-    tmp[MSGSIZE_MAX - 2] = '\n';
-    tmp[MSGSIZE_MAX - 1] = 0;
-    char *text = tmp;
+    bool use_tmp = lev == MSGL_STATUS || lev == MSGL_STATS;
 
-    print_msg_on_terminal(log, lev, text);
-    write_msg_to_buffers(log, lev, text);
-    dump_stats(log, lev, text);
+    struct mp_log_root *root = log->root;
+    char *text = use_tmp ? tmp : root->buffer;
+    int len = use_tmp ? 0 : strlen(text);
+
+    if (vsnprintf(text + len, MSGSIZE_MAX - len, format, va) < 0)
+        snprintf(text + len, MSGSIZE_MAX - len, "[fprintf error]\n");
+    text[MSGSIZE_MAX - 2] = '\n';
+    text[MSGSIZE_MAX - 1] = 0;
+
+    if (lev == MSGL_STATS) {
+        dump_stats(log, lev, text);
+    } else if (lev == MSGL_STATUS && !test_terminal_level(log, lev)) {
+        /* discard */
+    } else {
+        if (lev == MSGL_STATUS && root->termosd)
+            prepare_status_line(root, text);
+
+        // Split away each line. Normally we require full lines; buffer partial
+        // lines if they happen.
+        while (1) {
+            char *end = strchr(text, '\n');
+            if (!end)
+                break;
+            char *next = &end[1];
+            char saved = next[0];
+            next[0] = '\0';
+            print_terminal_line(log, lev, text);
+            write_msg_to_buffers(log, lev, text);
+            next[0] = saved;
+            text = next;
+        }
+
+        if (lev == MSGL_STATUS) {
+            if (text[0]) {
+                len = strlen(text);
+                if (len < MSGSIZE_MAX - 1) {
+                    text[len] = root->termosd ? '\r' : '\n';
+                    text[len + 1] = '\0';
+                }
+                print_terminal_line(log, lev, text);
+            }
+        } else {
+            int leftover = strlen(text);
+            memmove(root->buffer, text, leftover + 1);
+        }
+    }
 
     pthread_mutex_unlock(&mp_msg_lock);
 }
@@ -410,7 +427,6 @@ void mp_msg_init(struct mpv_global *global)
     struct mp_log_root *root = talloc_zero(NULL, struct mp_log_root);
     *root = (struct mp_log_root){
         .global = global,
-        .header = true,
         .reload_counter = ATOMIC_VAR_INIT(1),
     };
 
@@ -588,6 +604,18 @@ const char *const mp_log_levels[MSGL_MAX + 1] = {
     [MSGL_STATS]        = "stats",
 };
 
+const int mp_mpv_log_levels[MSGL_MAX + 1] = {
+    [MSGL_FATAL]        = MPV_LOG_LEVEL_FATAL,
+    [MSGL_ERR]          = MPV_LOG_LEVEL_ERROR,
+    [MSGL_WARN]         = MPV_LOG_LEVEL_WARN,
+    [MSGL_INFO]         = MPV_LOG_LEVEL_INFO,
+    [MSGL_STATUS]       = 0, // never used
+    [MSGL_V]            = MPV_LOG_LEVEL_V,
+    [MSGL_DEBUG]        = MPV_LOG_LEVEL_DEBUG,
+    [MSGL_TRACE]        = MPV_LOG_LEVEL_TRACE,
+    [MSGL_STATS]        = 0, // never used
+};
+
 int mp_msg_split_msglevel(struct bstr *s, struct bstr *out_mod, int *out_level)
 {
     if (s->len == 0)
