@@ -22,17 +22,21 @@
 
 #include "common/msg.h"
 #include "input/input.h"
-#include "input/event.h"
-#include "input/keycodes.h"
 
-#include "osdep/macosx_application_objc.h"
+#import "osdep/macosx_application_objc.h"
 #include "osdep/macosx_compat.h"
+#import "osdep/macosx_events_objc.h"
+#include "osdep/threads.h"
 
 #define MPV_PROTOCOL @"mpv://"
 
 static pthread_t playback_thread_id;
 
-@interface Application (PrivateMethods)
+@interface Application ()
+{
+    EventsResponder *_eventsResponder;
+}
+
 - (NSMenuItem *)menuItemWithParent:(NSMenu *)parent
                              title:(NSString *)title
                             action:(SEL)selector
@@ -61,17 +65,14 @@ Application *mpv_shared_app(void)
 @synthesize argumentsList = _arguments_list;
 @synthesize willStopOnOpenEvent = _will_stop_on_open_event;
 
-@synthesize inputContext = _input_context;
-@synthesize eventsResponder = _events_responder;
 @synthesize menuItems = _menu_items;
-@synthesize input_ready = _input_ready;
 
 - (void)sendEvent:(NSEvent *)event
 {
     [super sendEvent:event];
 
-    if (self.inputContext)
-        mp_input_wakeup(self.inputContext);
+    if (_eventsResponder.inputContext)
+        mp_input_wakeup(_eventsResponder.inputContext);
 }
 
 - (id)init
@@ -80,14 +81,8 @@ Application *mpv_shared_app(void)
         self.menuItems = [[[NSMutableDictionary alloc] init] autorelease];
         self.files = nil;
         self.argumentsList = [[[NSMutableArray alloc] init] autorelease];
-        self.eventsResponder = [[[EventsResponder alloc] init] autorelease];
+        _eventsResponder = [EventsResponder sharedInstance];
         self.willStopOnOpenEvent = NO;
-        self.input_ready = [[[NSCondition alloc] init] autorelease];
-
-        [NSEvent addLocalMonitorForEventsMatchingMask:NSKeyDownMask|NSKeyUpMask
-                                              handler:^(NSEvent *event) {
-            return [self.eventsResponder handleKey:event];
-        }];
 
         NSAppleEventManager *em = [NSAppleEventManager sharedAppleEventManager];
         [em setEventHandler:self
@@ -169,9 +164,10 @@ Application *mpv_shared_app(void)
 
 - (void)stopMPV:(char *)cmd
 {
-    if (self.inputContext) {
-        mp_cmd_t *cmdt = mp_input_parse_cmd(self.inputContext, bstr0(cmd), "");
-        mp_input_queue_cmd(self.inputContext, cmdt);
+    struct input_ctx *inputContext = _eventsResponder.inputContext;
+    if (inputContext) {
+        mp_cmd_t *cmdt = mp_input_parse_cmd(inputContext, bstr0(cmd), "");
+        mp_input_queue_cmd(inputContext, cmdt);
     } else {
         terminate_cocoa_application();
     }
@@ -274,23 +270,9 @@ Application *mpv_shared_app(void)
     }
 }
 
-
-- (void)handleFilesArray:(NSArray *)files
-{
-    size_t num_files  = [files count];
-    char **files_utf8 = talloc_array(NULL, char*, num_files);
-    [files enumerateObjectsUsingBlock:^(id obj, NSUInteger i, BOOL *_){
-        char *filename = (char *)[obj UTF8String];
-        size_t bytes   = [obj lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
-        files_utf8[i]  = talloc_memdup(files_utf8, filename, bytes + 1);
-    }];
-    mp_event_drop_files(self.inputContext, num_files, files_utf8);
-    talloc_free(files_utf8);
-}
-
 - (void)handleFiles
 {
-    [self handleFilesArray:self.files];
+    [_eventsResponder handleFilesArray:self.files];
 }
 @end
 
@@ -302,6 +284,7 @@ struct playback_thread_ctx {
 
 static void *playback_thread(void *ctx_obj)
 {
+    mpthread_set_name("playback core (OSX)");
     @autoreleasepool {
         struct playback_thread_ctx *ctx = (struct playback_thread_ctx*) ctx_obj;
         ctx->mpv_main(*ctx->argc, *ctx->argv);
@@ -322,10 +305,7 @@ int cocoa_main(mpv_main_fn mpv_main, int argc, char *argv[])
         macosx_finder_args_preinit(&argc, &argv);
         pthread_create(&playback_thread_id, NULL, playback_thread, &ctx);
 
-        [mpv_shared_app().input_ready lock];
-        while (!mpv_shared_app().inputContext)
-            [mpv_shared_app().input_ready wait];
-        [mpv_shared_app().input_ready unlock];
+        [[EventsResponder sharedInstance] waitForInputContext];
 
         cocoa_run_runloop();
 
@@ -380,14 +360,6 @@ void cocoa_stop_runloop(void)
     cocoa_post_fake_event();
 }
 
-void cocoa_set_input_context(struct input_ctx *input_context)
-{
-    [mpv_shared_app().input_ready lock];
-    mpv_shared_app().inputContext = input_context;
-    [mpv_shared_app().input_ready signal];
-    [mpv_shared_app().input_ready unlock];
-}
-
 void cocoa_post_fake_event(void)
 {
     NSEvent* event = [NSEvent otherEventWithType:NSApplicationDefined
