@@ -18,6 +18,7 @@
 
 #include <stddef.h>
 #include <stdbool.h>
+#include <pthread.h>
 #include <assert.h>
 
 #include "config.h"
@@ -25,11 +26,14 @@
 
 #include "osdep/io.h"
 #include "osdep/timer.h"
+#include "osdep/threads.h"
 
 #include "common/msg.h"
 #include "options/options.h"
 #include "options/m_property.h"
+#include "options/m_config.h"
 #include "common/common.h"
+#include "common/global.h"
 #include "common/encode.h"
 #include "common/playlist.h"
 #include "input/input.h"
@@ -61,12 +65,12 @@ double rel_time_to_abs(struct MPContext *mpctx, struct m_rel_time t)
         if (t.pos >= 0) {
             return start + t.pos;
         } else {
-            if (length != 0)
+            if (length >= 0)
                 return MPMAX(start + length + t.pos, 0.0);
         }
         break;
     case REL_TIME_PERCENT:
-        if (length != 0)
+        if (length >= 0)
             return start + length * (t.pos / 100.0);
         break;
     case REL_TIME_CHAPTER:
@@ -185,10 +189,14 @@ void error_on_track(struct MPContext *mpctx, struct track *track)
             MP_INFO(mpctx, "Audio: no audio\n");
         if (track->type == STREAM_VIDEO)
             MP_INFO(mpctx, "Video: no video\n");
-        if (!mpctx->current_track[0][STREAM_AUDIO] &&
-            !mpctx->current_track[0][STREAM_VIDEO])
-            mpctx->stop_play = PT_NEXT_ENTRY;
-        mpctx->error_playing = true;
+        if (mpctx->opts->stop_playback_on_init_failure ||
+            (!mpctx->current_track[0][STREAM_AUDIO] &&
+             !mpctx->current_track[0][STREAM_VIDEO]))
+        {
+            mpctx->stop_play = PT_ERROR;
+            if (mpctx->error_playing >= 0)
+                mpctx->error_playing = MPV_ERROR_NOTHING_TO_PLAY;
+        }
         mpctx->sleeptime = 0;
     }
 }
@@ -237,3 +245,65 @@ void merge_playlist_files(struct playlist *pl)
     playlist_add_file(pl, edl);
     talloc_free(edl);
 }
+
+// Create a talloc'ed copy of mpctx->global. It contains a copy of the global
+// option struct. It still just references some things though, like mp_log.
+// The main purpose is letting threads access the option struct without the
+// need for additional synchronization.
+struct mpv_global *create_sub_global(struct MPContext *mpctx)
+{
+    struct mpv_global *new = talloc_ptrtype(NULL, new);
+    struct m_config *new_config = m_config_dup(new, mpctx->mconfig);
+    *new = (struct mpv_global){
+        .log = mpctx->global->log,
+        .opts = new_config->optstruct,
+    };
+    return new;
+}
+
+struct wrapper_args {
+    struct MPContext *mpctx;
+    void (*thread_fn)(void *);
+    void *thread_arg;
+    pthread_mutex_t mutex;
+    bool done;
+};
+
+static void *thread_wrapper(void *pctx)
+{
+    struct wrapper_args *args = pctx;
+    mpthread_set_name("opener");
+    args->thread_fn(args->thread_arg);
+    pthread_mutex_lock(&args->mutex);
+    args->done = true;
+    pthread_mutex_unlock(&args->mutex);
+    mp_input_wakeup(args->mpctx->input); // this interrupts mp_idle()
+    return NULL;
+}
+
+// Run the thread_fn in a new thread. Wait until the thread returns, but while
+// waiting, process input and input commands.
+int mpctx_run_non_blocking(struct MPContext *mpctx, void (*thread_fn)(void *arg),
+                           void *thread_arg)
+{
+    struct wrapper_args args = {mpctx, thread_fn, thread_arg};
+    pthread_mutex_init(&args.mutex, NULL);
+    bool success = false;
+    pthread_t thread;
+    if (pthread_create(&thread, NULL, thread_wrapper, &args))
+        goto done;
+    while (!success) {
+        mp_idle(mpctx);
+
+        if (mpctx->stop_play)
+            mp_cancel_trigger(mpctx->playback_abort);
+
+        pthread_mutex_lock(&args.mutex);
+        success |= args.done;
+        pthread_mutex_unlock(&args.mutex);
+    }
+    pthread_join(thread, NULL);
+done:
+    pthread_mutex_destroy(&args.mutex);
+    return success ? 0 : -1;
+}
