@@ -506,8 +506,7 @@ int dvb_set_channel(stream_t *stream, int card, int n)
         priv->fd = priv->dvr_fd;
         MP_VERBOSE(stream, "DVB_SET_CHANNEL: new channel name=%s, card: %d, channel %d\n", channel->name, card, n);
 
-        stream->buf_pos = stream->buf_len = 0;
-        stream->pos = 0;
+        stream_drop_buffers(stream);
 
         if(channel->freq != priv->last_freq)
                 if (! dvb_tune(priv, channel->freq, channel->pol, channel->srate, channel->diseqc, channel->tone,
